@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  * Originally based on rhythmbox/lib/rb-debug.c
  *
  * Licensed under the GNU General Public License Version 2
@@ -33,25 +33,36 @@
 
 #include "gpm-common.h"
 #include "gpm-debug.h"
-#include "gpm-common.h"
 
 static gboolean is_init = FALSE;	/* if we are initialised */
 static gboolean do_verbose = FALSE;	/* if we should print out debugging */
+static GSList *list = NULL;
+static gchar va_args_buffer [1025];
+
+/**
+ * gpm_add_debug_option:
+ **/
+void
+gpm_add_debug_option (const gchar *option)
+{
+	/* adding debug option to list */
+	list = g_slist_prepend (list, (gpointer) option);
+}
 
 /**
  * gpm_print_line:
  **/
 static void
-gpm_print_line (const char *func,
-		const char *file,
-		const int   line,
-		const char *buffer)
+gpm_print_line (const gchar *func,
+		const gchar *file,
+		const int    line,
+		const gchar *buffer)
 {
-	char   *str_time;
+	gchar   *str_time;
 	time_t  the_time;
 
 	time (&the_time);
-	str_time = g_new0 (char, 255);
+	str_time = g_new0 (gchar, 255);
 	strftime (str_time, 254, "%H:%M:%S", localtime (&the_time));
 
 	fprintf (stderr, "[%s] %s:%d (%s):\t %s\n",
@@ -59,52 +70,89 @@ gpm_print_line (const char *func,
 	g_free (str_time);
 }
 
+/**
+ * gpm_debug_strcmp_func
+ * @a: Pointer to the data to test
+ * @b: Pointer to a cookie to compare
+ * Return value: 0 if cookie matches
+ **/
+static gint
+gpm_debug_strcmp_func (gconstpointer a, gconstpointer b)
+{
+	gchar *aa = (gchar *) a;
+	gchar *bb = (gchar *) b;
+	return strcmp (aa, bb);
+}
+
+/**
+ * gpm_debug_in_options:
+ **/
+static gboolean
+gpm_debug_in_options (const gchar *file)
+{
+	GSList *node;
+	gchar *name;
+	guint8 len;
+
+	/* get rid of the "gpm-" prefix */
+	name = strdup (&file[4]);
+
+	/* get rid of .c */
+	len = strlen (name);
+	if (len>2) {
+		name[len-2] = '\0';
+	}
+
+	/* find string in list */
+	node = g_slist_find_custom (list, name, gpm_debug_strcmp_func);
+	g_free (name);
+	return (node != NULL);
+}
+
 /**
  * gpm_debug_real:
  **/
 void
-gpm_debug_real (const char *func,
-		const char *file,
-		const int   line,
-		const char *format, ...)
+gpm_debug_real (const gchar *func,
+		const gchar *file,
+		const int    line,
+		const gchar *format, ...)
 {
 	va_list args;
-	char    buffer [1025];
 
-	if (! do_verbose) {
+	if (do_verbose == FALSE && gpm_debug_in_options (file) == FALSE) {
 		return;
 	}
 
 	va_start (args, format);
-	g_vsnprintf (buffer, 1024, format, args);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
 	va_end (args);
 
-	gpm_print_line (func, file, line, buffer);
+	gpm_print_line (func, file, line, va_args_buffer);
 }
 
 /**
  * gpm_warning_real:
  **/
 void
-gpm_warning_real (const char *func,
-		  const char *file,
-		  const int   line,
-		  const char *format, ...)
+gpm_warning_real (const gchar *func,
+		  const gchar *file,
+		  const int    line,
+		  const gchar *format, ...)
 {
 	va_list args;
-	char    buffer [1025];
 
-	if (! do_verbose) {
+	if (do_verbose == FALSE) {
 		return;
 	}
 
 	va_start (args, format);
-	g_vsnprintf (buffer, 1024, format, args);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
 	va_end (args);
 
 	/* do extra stuff for a warning */
 	fprintf (stderr, "*** WARNING ***\n");
-	gpm_print_line (func, file, line, buffer);
+	gpm_print_line (func, file, line, va_args_buffer);
 }
 
 /**
@@ -121,6 +169,19 @@ gpm_bugzilla (void)
 		 "GNOME Power Manager", GPM_BUGZILLA_URL);
 }
 
+/**
+ * gpm_syslog:
+ * @format: This va format string, e.g. ("test %s", hello)
+ *
+ * Logs some text to the syslog, usually in /var/log/messages
+ **/
+static void
+gpm_syslog_internal (const gchar *string)
+{
+	fprintf (stderr, "Saving to syslog: %s", string);
+	syslog (LOG_NOTICE, "(%s) %s", g_get_user_name (), string);
+}
+
 /**
  * gpm_syslog:
  * @format: This va format string, e.g. ("test %s", hello)
@@ -128,17 +189,15 @@ gpm_bugzilla (void)
  * Logs some text to the syslog, usually in /var/log/messages
  **/
 void
-gpm_syslog (const char *format, ...)
+gpm_syslog (const gchar *format, ...)
 {
 	va_list args;
-	char    buffer [1025];
 
 	va_start (args, format);
-	g_vsnprintf (buffer, 1024, format, args);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
 	va_end (args);
 
-	gpm_debug ("Saving to syslog: %s", buffer);
-	syslog (LOG_ALERT, "%s", buffer);
+	gpm_syslog_internal (va_args_buffer);
 }
 
 /**
@@ -166,21 +225,20 @@ gpm_debug_init (gboolean debug)
  * @content: The content to show, e.g. "No icons detected"
  *
  * Shows a gtk critical error and logs to syslog.
- * NOTE: we will loose memory, but since this program is a critical error
+ * NOTE: we will lose memory, but since this program is a critical error
  * that is the least of our problems...
  **/
 void
-gpm_critical_error (const char *format, ...)
+gpm_critical_error (const gchar *format, ...)
 {
 	va_list args;
-	char    buffer [1025];
+	GtkWidget *dialog;
 
 	va_start (args, format);
-	g_vsnprintf (buffer, 1024, format, args);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
 	va_end (args);
 
-	gpm_syslog ("Critical error: %s", buffer);
-	GtkWidget *dialog;
+	gpm_syslog_internal (va_args_buffer);
 	dialog = gtk_message_dialog_new_with_markup (NULL,
 						     GTK_DIALOG_MODAL,
 						     GTK_MESSAGE_WARNING,
@@ -188,7 +246,7 @@ gpm_critical_error (const char *format, ...)
 						     "<span size='larger'><b>%s</b></span>",
 						     GPM_NAME);
 	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
-						    buffer);
+						    va_args_buffer);
 	/* we close the gtk loop when the user clicks ok */
 	g_signal_connect_swapped (dialog,
 				  "response",
@@ -203,8 +261,9 @@ gpm_critical_error (const char *format, ...)
 void
 gpm_debug_shutdown (void)
 {
-	if (! is_init)
+	if (! is_init) {
 		return;
+	}
 
 	gpm_debug ("Shutting down debugging");
 	is_init = FALSE;
